#!/bin/bash
# shellcheck disable=SC1091 # Cannot resolve paths in variables

PROJECT_HOME="$(realpath -s --relative-to . "$(dirname "$0")/..")" || exit
. "$PROJECT_HOME/scripts/util.sh" || exit

TYPE="$1"

[[ -z "$TYPE" ]] && error $"Usage: $0 <type>"

case "$TYPE" in
development | release) ;;
*) error $"$TYPE must be 'development' or 'release'" ;;
esac

get_project_name
get_arch
get_project_type
get_version

CONTAINERS_DIR="$PROJECT_HOME/swu/app/etc/atmark/containers"
APP_DESC_FILE="$PROJECT_HOME/swu/app.desc"
APP_DESC_TMPL_FILE="$PROJECT_HOME/swu/app.desc.tmpl"
USER_DESC_FILE="$PROJECT_HOME/config/app.desc"
USER_CONFIG_FILE="$PROJECT_HOME/config/app.conf"
CONTAINER_CONFIG_FILE="$CONTAINERS_DIR/$PROJECT.conf"
OUT_SWU="$PROJECT_HOME/$TYPE.swu"
TRANSFER_FILES_DIR="$PROJECT_HOME/swu/dest"

gen_swu() {
	declare -a desc_files=(
		"$@"
	)

	[ -e "$APP_DESC_TMPL_FILE" ] ||
		error $"$APP_DESC_TMPL_FILE not found, delete swu/app.desc ?"

	# MODE is used by flutter app
	[[ $TYPE == "development" ]] && MODE="debug" || MODE="release"

	# variable expand app.desc template and copy its installed name.
	# shellcheck disable=SC2153 # VERSION is gotten by util.sh.
	sed -e "s/{{PROJECT}}/$PROJECT/g" -e "s/{{VERSION}}/$VERSION/g" \
		-e "s/{{MODE}}/$MODE/g" "$APP_DESC_TMPL_FILE" >"$APP_DESC_FILE"

	# Increase version to allow upgrade
	mkswu --update-version "$APP_DESC_FILE" ||
		error $"Could not increase $APP_DESC_FILE version -- is mkswu up to date?"
	# Increase version of info.json too
	version=$(awk -F= '$1 == "swdesc_option version" { print $2 }' "$APP_DESC_FILE")
	[[ -n "$version" ]] || error $"version is empty. of swdesc_option version delete swu/app.desc ?"
	[[ "$version" != *" "* ]] ||
		error $"version:$version is syntax error. change swdesc_option version of swu/app.desc ?"
	jq -r ".version |= \"$version\"" "${INFO_FILE}" >tmpfile && mv tmpfile "${INFO_FILE}"

	# This variable is used in the desc file
	export ARCH

	case "$TYPE" in
	development)
		# enable sshd on armadillo
		desc_files+=(
			"$PROJECT_HOME/swu/enable_sshd.desc"
		)
		# and also remove known hosts in case of reinstall
		rm -f "$PROJECT_HOME/config/ssh_known_hosts"
		;;
	esac

	# merge all desc contents together in order to share
	# same version, which would be reset with different files
	cat "${desc_files[@]}" |
		mkswu -o "$OUT_SWU" - ||
		exit

	rm -f "$CONTAINER_CONFIG_FILE"
}

confirm_crlf "$PROJECT_HOME"/app

case "$PROJECT_TYPE" in
"shell"|"python")
	# Copy app/src to swu/dest if shell or python project.
	rm -rf "$TRANSFER_FILES_DIR"
	mkdir -p "$TRANSFER_FILES_DIR"
	cp -a "${PROJECT_HOME}/app/"* "$TRANSFER_FILES_DIR"
	;;&
"c")
	# if execting c lang project, exec make command.
	"$PROJECT_HOME"/scripts/build_container_image.sh build || exit
	build_project || exit
	;;&
"gateway")
	gen_swu "$APP_DESC_FILE"
	;;
"flutter")
	# change the name of the application to be launched to the project name
	LAUNCH_FILE="$PROJECT_HOME"/container/resources_flutter/bin/flutter_launch
	APP_NAME=$(yq -r '.name' app/pubspec.yaml)
	sed -e "s/{{APP_NAME}}/$APP_NAME/g" \
		"$PROJECT_HOME/swu/flutter_launch.tmpl" >"$LAUNCH_FILE"
	chmod +x "$LAUNCH_FILE"
	;&
*)
	"$PROJECT_HOME/scripts/build_container_image.sh" || exit
	# Generate container config from template
	mkdir -p "$CONTAINERS_DIR"
	sed -e "s/{{PROJECT}}/$PROJECT/g" "$USER_CONFIG_FILE" >"$CONTAINER_CONFIG_FILE"
	gen_swu "$APP_DESC_FILE" "$USER_DESC_FILE"
	;;
esac

echo $"Install $OUT_SWU on Armadillo next"
